% 1. This file documents the continuation code for travelling waves in neural
% fields with linear adaptation with and without travel delays. Accompanies
% paper by Meijer HGE and Coombes S, "Travelling waves in models of neural
% tissue: from localised structures to periodic waves" to appear in EPJ
% Nonlinear Biomedical Physics 2014.
% 2. There is also simulation code, see simadapt.m.
% 3. This is a research code, some handles must be set manually, i.e. to
% switch between travel delays and no delays and between exponential and
% Gaussian connectivity, look into adapt_discrhs.m.
% 4. Any part may be used and changed if this source (paper) is referred
% to. No warrany is given, questions are welcome. To use this file, the
% basics of numerical continuation are assumed. In particular, I cannot
% garantuee the direction of the computed curve as this depends on matlab!
% Manual adjustment of the tangent direction may be needed.
% Last edit 17-2-2014 by H.G.E. Meijer

%If possible, use parallel toolbox.
% matlabpool open 7 %On a double-threaded quad-core this leaves one process for monitoring.

%The continuation needs 
%1. an inital guess of the travelling wave (periodic orbit). The initial
%guess can be the final profile obtained from a simulatio. However in order
%to execute this file stand-alone, a initial profile is given below. Hence, 
%a few more Newton corrections are needed. For didactical/speed reasons a
%low order discretization is chosen. The paper uses N=11-13 for accuracy.
%2. a vector of system parameters pars with the following 8 components.
%period=pars(1);cc=pars(2);vv=pars(3);beta=pars(4);
%theta=pars(5);kappa=pars(6);tau=pars(7);sigma=pars(8);
%3. index/vector with two elements specifying the parameters to be varied.

par1=[30 0.80 20 10 .3 .9 7 1];N=7;
u0=.3+.4*cos(2*pi*(0:2^N-1)/2^N);
a0=.2+.15*cos(.1+2*pi*(0:2^N-1)/2^N);
x1=[u0 a0]';ind=[1;2];
sol1=adaptsolver(x1,par1,ind);
figure(1);clf(1);hold on;
plot(sol1(end-1,:),sol1(end,:));
%Now extend the continuation
x2=sol1(1:end-2,end);par2=par1;par2(ind)=sol1(end-1:end,end);
sol2=adaptsolver(x2,par2,[1;2]);
plot(sol2(end-1,:),sol2(end,:));

%This does not look good, i.e. the lower branch is monotonically decreasing
%instead of converging to some limit value. So we refine the solution twice:
x1=sol1(1:end-2,1);par3=par1;par3([1 2])=sol1(end-1:end,1);
x3=zeros(2*size(x1,1),1);x3(1:2:end)=x1;
x3(2:2:end)=(x3(1:2:end-1)+[x3(3:2:end-1); x3(1)])/2;x1=x3;
x3=zeros(2*size(x1,1),1);x3(1:2:end)=x1;
x3(2:2:end)=(x3(1:2:end-1)+[x3(3:2:end-1); x3(1)])/2;
sol3=adaptsolver(x3,par3,ind);
plot(sol3(end-1,:),sol3(end,:),'Color','Black','Linewidth',2);

%The resulting dispersion curve should be similar to the figure in the zip-file.