function sol=adaptsolver(uu,pars,ind)
if (size(pars,1)==1)
  pars=pars';
end
%Continuation Settings
Stepsize=.5;Steps=200;Nx=size(uu,1);
sol=zeros(Nx+2,Steps);

%Given an initial point we need to correct it first.
ii=0;error=1;u0=uu;
while(norm(error)>5e-5 && ii<26)
  error=adapt_discrhs(uu,u0,pars);
  JAC=sparse(jacobian(uu,u0,pars,ind));
  JAC1=JAC(:,[1:Nx end]);
  %NEWTON STEP;
  xx=[uu;pars(ind(2))]-JAC1\error;uu=xx(1:end-1);pars(ind(2))=xx(end);
  ii=ii+1;norm(error)
end
disp('First point found...');
format compact;format short;
sol(:,1)=[uu;pars(ind)];
jj=1;disp([jj ii sol(end-1:end,jj)']);
tangent=[JAC;zeros(1,Nx+1) 1]\[zeros(Nx+1,1);1];
%Comment next line to compute the other direction of the solution branch 
% tangent=-tangent/norm(tangent);
tangent=tangent/norm(tangent);
JAC=zeros(Nx+2,Nx+2);

%% %% %% CONTINUATION PART
for jj=2:Steps
conv=0;
while(conv~=1)
%Predict new point
  ii=0;error=1;
  uu=sol(1:Nx,jj-1)+Stepsize*tangent(1:end-2);
  pars(ind)=sol(end-1:end,jj-1)+Stepsize*tangent(end-1:end);
%Find new point on the curve
  while(norm(error)>5e-5 && ii<4)
    error=adapt_discrhs(uu,sol(1:Nx,jj-1),pars);
    JAC(1:end-1,:)=jacobian(uu,sol(1:Nx,jj-1),pars,ind);
    %PSEUDO_ARCLENGTH CONDITION:
    error(end+1)=tangent'*([uu;pars(ind)]-sol(:,jj-1))-Stepsize;
    JAC(end,:)=tangent;  %norm(error)  ;  
    %NEWTON STEP;
    xx=[uu;pars(ind)]-JAC\error;uu=xx(1:end-2);pars(ind)=xx(end-1:end);
    ii=ii+1;
  end
%Stepsize adaptation
  if(ii<4)
    conv=1;
    Stepsize=min(.25,1.3*Stepsize);
  else
    Stepsize=Stepsize/2;
  end
  if(Stepsize<1e-5)
    disp('Stepsize too small...');
    return;
  end
end
%update solution and tangent vector;
  sol(:,jj)=xx;
  tangent=JAC\[zeros(Nx+1,1);1];tangent=tangent/norm(tangent);
  disp([jj ii sol(end-1:end,jj)']);
end

function JAC=jacobian(uu,u0,pars,ind)
%Determine Jacobian of defining system
Nx=size(uu,1);JAC=zeros(Nx+1,Nx+2);eps=1e-5;
%IF POSSIBLE USE THE PARALLEL TOOLBOX: already on a quadcore pc this speeds up.
parfor i=1:Nx
  pp=uu;pp(i)=pp(i)+eps;
  r1=adapt_discrhs(pp,u0,pars);
  pp=uu;pp(i)=pp(i)-eps;
  r2=adapt_discrhs(pp,u0,pars);
  JAC(:,i)=(r1-r2)/(2*eps);
end
par=pars;par(ind(1))=par(ind(1))+eps;
r1=adapt_discrhs(uu,u0,par);
par=pars;par(ind(1))=par(ind(1))-eps;
r2=adapt_discrhs(uu,u0,par);
JAC(:,end-1)=(r1-r2)/(2*eps);
par=pars;par(ind(2))=par(ind(2))+eps;
r1=adapt_discrhs(uu,u0,par);
par=pars;par(ind(2))=par(ind(2))-eps;
r2=adapt_discrhs(uu,u0,par);
JAC(:,end)=(r1-r2)/(2*eps);
