% Code to simulate travelling waves in neural fields with linear adaptation
% with and without travel delays. Accompanies paper by Meijer HGE and
% Coombes S, "Travelling waves in models of neural tissue: from localised
% structures to periodic waves" to appear in EPJ Nonlinear Biomedical
% Physics 2014. There is also continuation code see README_ADAPT.m
% Created 14-2-2014 H.G.E. Meijer
%
% (Un)comment the appropiate parts to change initial conditions/history and
% connectivity.

% Spatial discretization
Nx=2^10;S=1;period=50; dx=period/Nx;x=dx*(0:Nx-1);
% Connectivity convolution by Fourier
kk=((0:Nx-1)- (Nx/2))*2*pi/period;FW=fftshift(S^2./(S^2+kk.^2));
% Gaussian: 
% ww=[exp(-(dx/2)^2*abs(-Nx/2:-1).^2) 1 exp(-(dx/2)^2*abs(1:Nx/2-1).^2)]/2/sqrt(pi);
% FW=fft(fftshift(ww*dx));

% % Initial conditions for right moving waves
beta=42; theta=.3; gain=0.75; tau=7;       
y0=[.2+1.3*cos(.5+2*pi*x/period) .1+.3*cos(2+2*pi*x/period)];NB=1;%1 Bump
% % Initial conditions for right moving waves/anti-pulses
beta=42; theta=.3; gain=0.6; tau=7;       
y0=[.2+1.3*cos(.5+2*pi*x/period) .1+.3*cos(2+2*pi*x/period)];NB=1;%1 Bump

% % Initial conditions for oscillatory disp curves
% beta=9; theta=.3; gain=0.75; tau=7;       
% y0=[.2+1.3*cos(.5+2*pi*x/period) .1+.3*cos(2+2*pi*x/period)];NB=1;%1 Bump
% y0=[.2+.4*cos(4*pi*x/period) .1+.15*cos(1.5+4*pi*x/period)];NB=2;%2 Bumps
% % Initial conditions for dispersion gap
% beta=8; theta=.3; gain=0.96; tau=7;   
% y0=[.2+1.3*cos(.5+2*pi*x/period) .1+.3*cos(2+2*pi*x/period)];NB=1;%1 Bump
% WAVETRAINS for ALPHA synapse, make changes in odeadapt as well!
% y0=[.2+1.3*cos(.5+20*pi*x/period) .1-.02*cos(2.5+8*pi*x/period)+.1*cos(2*pi*x/period)+.3*cos(2+20*pi*x/period)];NB=1;%1 Bump
% y0=[.1+1.3*cos(.5+2*pi*x/period) .4+1.3*cos(1.5+2*pi*x/period) .1+.3*cos(2+2*pi*x/period)];NB=1;%1 Bump

% % Simulation
tfinal=150;options = odeset('RelTol',1e-4,'AbsTol',1e-4,'MaxStep',.25);tic
[t,y] = ode45(@odeadapt,[0:.25:tfinal],y0, options,...
        FW,beta,theta,gain,tau);toc
% Make a figure, (1) space-time plot (2) final profile
figure1=figure(1);clf(1);axes1 = axes('Parent',figure1,'Layer','top');
xlim(axes1,x([1 end]));ylim(axes1,t([1 end]));box(axes1,'on');hold(axes1,'all');
imagesc(x([1 end]),t([1 end]),y(:,1:Nx))
xlabel('space');ylabel('time');
figure(2);clf(2);hold on;
plot(x,y(end,1:Nx),x,y(end,Nx+1:end))


%% %% %% %% %% %% %% %% %% %% %% %% %% %% %% %%
% With delays; code follows the implementation of Chap 9.2, Listing 9.2.5
% from the Ph.D. Thesis of N.A. Venkov, 2008, University of Nottingham 
%
% %System parameters
% beta=20; theta=.3; gain=0.95; tau=5;v=30;
% %Spatial discretization, connectivity and delays
% Nx=2^8;period=50;dx=period/Nx;x=dx*(0:Nx-1);
% % Exponential connectivity, cutoff where WW(x=20)=1e-12, hence numerically negligible
% WW=dx*exp(-dx*abs(0:ceil(20/dx)))/2;lags=(1:ceil(20/dx))*dx/v;
% % Gaussian connectivity, fewer points are needed for cutoff.
% % W1=dx*exp(-(dx/2)^2*abs(0:ceil(10/dx)).^2)/2/sqrt(pi);lags=(1:ceil(10/dx))*dx/v;
% WW([1 end])=WW([1 end])/2;%Trapezoidal rule
% 
% tfinal=120;options = ddeset('RelTol',1e-6,'AbsTol',1e-6);tic
% sol = dde23('ddeadapt', lags, @historyadapt, [0,tfinal], options,...
%         WW,Nx, beta, theta, gain,tau);toc
% % % Make a figure, (3) space-time plot
% figure(3);imagesc(x,sol.x(end:-1:1),sol.y(1:Nx,end:-1:1)');